package com.jiuqi.grid;

import com.jiuqi.util.Base64;
import com.jiuqi.util.MemStream;
import com.jiuqi.util.Stream;
import com.jiuqi.util.StreamException;
import com.jiuqi.util.StringUtils;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.Stack;

/* loaded from: com_jiuqi_grid.jar:com/jiuqi/grid/GridData.class */
public final class GridData implements Serializable, Cloneable {
    private IntList cellColors;
    private IntList fontColors;
    private IntList edgeColors;
    private GridFontNames fontNames;
    private IntList fontSizes;
    private IntList colWidths;
    private IntList rowHeights;
    private BitList colVisible;
    private BitList rowVisible;
    private BitList hideDataRow;
    private BitList hideDataCol;
    private BitList colAutoSize;
    private BitList rowAutoSize;
    private Matrix values;
    private Matrix cells;
    private GridFieldList rects;
    private GridCell cell;
    private PropList propList;
    private Matrix objects;
    private int scrollTopCol;
    private int scrollTopRow;
    private int scrollBottomCol;
    private int scrollBottomRow;
    private int curCol;
    private int curRow;
    private boolean useFontSize;
    private LabelItems labels;
    private Matrix cellCSS;
    private String gridDataClass;
    private String referenceBase;
    private Matrix cellScript;
    private int horzOffset;
    private int vertOffset;
    private boolean horzCenter;
    private boolean vertCenter;
    private ExtendedDatas extDatas;
    private int options;
    private Set modifiedPropIndexRecycle;
    private boolean modifying;
    private int colCountPerPage;
    private int rowCountPerPage;
    private boolean rowPagingFirst;
    public static final int GRID_AUTOMERGE = 1;
    public static final int GRID_AUTOSTYLE = 2;
    public static final int GRID_DYNAMICSTORE = 4;
    public static final int GRID_FULLSTYLE = 8;

    public GridData() {
        this.scrollTopCol = -1;
        this.scrollTopRow = -1;
        this.scrollBottomCol = -1;
        this.scrollBottomRow = -1;
        this.curCol = -1;
        this.curRow = -1;
        this.labels = new LabelItems();
        this.cellCSS = null;
        this.cellScript = null;
        this.modifying = false;
        this.colCountPerPage = 0;
        this.rowCountPerPage = 0;
        this.rowPagingFirst = true;
        this.cellColors = GridConsts.create64ColorsPallette();
        this.fontColors = GridConsts.create64ColorsPallette();
        this.edgeColors = GridConsts.create16ColorsPallette();
        this.fontNames = new GridFontNames(64);
        this.fontNames.addFont(Font.DefaultFontName);
        this.fontSizes = new IntList();
        this.fontSizes.add(9);
        this.colVisible = new BitList(true);
        this.rowVisible = new BitList(true);
        this.colAutoSize = new BitList(false);
        this.rowAutoSize = new BitList(false);
        this.hideDataRow = new BitList(false);
        this.hideDataCol = new BitList(false);
        this.colWidths = new IntList();
        this.rowHeights = new IntList();
        this.values = new Matrix();
        this.cells = new Matrix();
        this.objects = new Matrix();
        this.rects = new GridFieldList(this);
        this.cell = new GridCell();
        this.propList = new PropList(20);
        this.propList.add(GridConsts.DEF_CELL_PROP);
        this.extDatas = new ExtendedDatas();
        setColCount(5);
        setRowCount(5);
        for (int i = 0; i < 5; i++) {
            GridCell cell = getCell(i, 0);
            setCell(i, 0, cell);
        }
        for (int i2 = 0; i2 < 5; i2++) {
            GridCell cell2 = getCell(0, i2);
            setCell(0, i2, cell2);
        }
        this.options = 7;
        setOptions(this.options);
    }

    public GridData(int colCount, int rowCount) {
        this();
        setColCount(colCount);
        setRowCount(rowCount);
    }

    public GridData(int options) {
        this();
        setOptions(options);
    }

    public GridData(int colCount, int rowCount, int options) {
        this(colCount, rowCount);
        setOptions(options);
    }

    public void beginModify() {
        if (isModifying()) {
            throw new IllegalStateException("无法对已经处于修改状态的GridData执行beginModify方法！");
        }
        if (this.modifiedPropIndexRecycle == null) {
            this.modifiedPropIndexRecycle = new HashSet();
        }
        this.modifying = true;
    }

    /* JADX WARN: Multi-variable type inference failed */
    /* JADX WARN: Type inference failed for: r0v0 */
    /* JADX WARN: Type inference failed for: r0v1, types: [java.lang.Throwable] */
    /* JADX WARN: Type inference failed for: r0v3 */
    public void secureBeginModify() {
        ?? r0 = this;
        synchronized (r0) {
            beginModify();
            r0 = r0;
        }
    }

    public boolean isModifying() {
        return this.modifying;
    }

    public void endModify() {
        rebuildPropIndexes(this.modifiedPropIndexRecycle);
        this.modifiedPropIndexRecycle.clear();
        this.modifying = false;
    }

    /* JADX WARN: Multi-variable type inference failed */
    /* JADX WARN: Type inference failed for: r0v0 */
    /* JADX WARN: Type inference failed for: r0v1, types: [java.lang.Throwable] */
    /* JADX WARN: Type inference failed for: r0v3 */
    public void secureEndModify() {
        ?? r0 = this;
        synchronized (r0) {
            endModify();
            r0 = r0;
        }
    }

    private void createCellCssMatrix() {
        if (this.cellCSS == null) {
            this.cellCSS = new Matrix();
            this.cellCSS.setSize(getColCount(), getRowCount());
        }
    }

    private void createCellScriptMatrix() {
        if (this.cellScript == null) {
            this.cellScript = new Matrix();
            this.cellScript.setSize(getColCount(), getRowCount());
        }
    }

    private boolean checkCol(int col, boolean isInsert) {
        return isInsert ? col > 0 && col <= getColCount() : col >= 0 && col < getColCount();
    }

    private boolean checkRow(int row, boolean isInsert) {
        return isInsert ? row > 0 && row <= getRowCount() : row >= 0 && row < getRowCount();
    }

    private void rebuildPropIndex(int col, int row, int oldIndex) {
        if (oldIndex < 0) {
            return;
        }
        if (this.modifying) {
            this.modifiedPropIndexRecycle.add(new Integer(oldIndex));
            return;
        }
        if (!this.cells.hasElement(new Integer(oldIndex))) {
            for (int x = 0; x < this.cells.getXCount(); x++) {
                for (int y = 0; y < this.cells.getYCount(); y++) {
                    Integer index = (Integer) this.cells.getElement(x, y);
                    if (index != null && index.intValue() > oldIndex) {
                        this.cells.setElement(x, y, new Integer(index.intValue() - 1));
                    }
                }
            }
            this.propList.remove(oldIndex);
        }
    }

    private void rebuildPropIndexes(Set bufferedIndex) {
        if (bufferedIndex.size() == 0 || this.propList.count() == 0) {
            return;
        }
        int[] propCase = new int[this.propList.count()];
        for (Object index : bufferedIndex) {
            if (!this.cells.hasElement(index)) {
                propCase[((Integer) index).intValue()] = -1;
            }
        }
        for (int ci = propCase.length - 1; ci >= 0; ci--) {
            if (propCase[ci] != 0) {
                this.propList.remove(ci);
            }
        }
        for (int ci2 = 1; ci2 < propCase.length; ci2++) {
            int i = ci2;
            propCase[i] = propCase[i] + propCase[ci2 - 1];
        }
        if (propCase[propCase.length - 1] == 0) {
            return;
        }
        for (int x = 0; x < this.cells.getXCount(); x++) {
            for (int y = 0; y < this.cells.getYCount(); y++) {
                Integer index2 = (Integer) this.cells.getElement(x, y);
                if (index2 != null && propCase[index2.intValue()] != 0) {
                    this.cells.setElement(x, y, new Integer(index2.intValue() + propCase[index2.intValue()]));
                }
            }
        }
    }

    public String getDataClass() {
        return this.gridDataClass;
    }

    public void setDataClass(String value) {
        this.gridDataClass = value;
    }

    public String getReferenceBase() {
        return this.referenceBase;
    }

    public void setReferenceBase(String value) {
        this.referenceBase = value;
    }

    public String getCellData(int col, int row) {
        return (String) this.values.getElement(col, row);
    }

    public String getCellData(String pos) {
        CellPostion p = new CellPostion(pos);
        return getCellData(p.getCol(), p.getRow());
    }

    public void setCellData(int col, int row, String value) {
        this.values.setElement(col, row, value);
    }

    public void setCellData(String pos, String value) {
        CellPostion p = new CellPostion(pos);
        setCellData(p.getCol(), p.getRow(), value);
    }

    public String getCellClass(int col, int row) {
        if (this.cellCSS == null) {
            return null;
        }
        return (String) this.cellCSS.getElement(col, row);
    }

    public void setCellClass(int col, int row, String value) {
        if (this.cellCSS == null) {
            if (value == null || value.length() == 0) {
                return;
            } else {
                createCellCssMatrix();
            }
        }
        this.cellCSS.setElement(col, row, value);
    }

    public boolean getUseFontSize() {
        return this.useFontSize;
    }

    public void setUserFontSize(boolean value) {
        this.useFontSize = value;
    }

    public String getCellScript(int col, int row) {
        if (this.cellScript == null) {
            return null;
        }
        return (String) this.cellScript.getElement(col, row);
    }

    public void setCellScript(int col, int row, String value) {
        if (this.cellScript == null) {
            if (value == null || value.length() == 0) {
                return;
            } else {
                createCellScriptMatrix();
            }
        }
        this.cellScript.setElement(col, row, value);
    }

    public int getCellStyleIndex(int col, int row) {
        Integer index = (Integer) this.cells.getElement(col, row);
        if (index == null) {
            return 0;
        }
        return index.intValue();
    }

    public GridCell[] getCellSytles() {
        if (this.propList.count() > 0) {
            GridCell[] ret = new GridCell[this.propList.count()];
            for (int i = 0; i < ret.length; i++) {
                ret[i] = new GridCell();
                ret[i].init(this, 0, 0, this.propList.get(i));
            }
            return ret;
        }
        return new GridCell[0];
    }

    public GridCell[] getCellStyles(int left, int top, int right, int bottom, boolean fullMode) {
        if (this.propList.count() == 0) {
            return new GridCell[0];
        }
        GridCell[] ret = new GridCell[this.propList.count()];
        for (int col = left; col <= right; col++) {
            for (int row = top; row <= bottom; row++) {
                if (StringUtils.isEmpty(getCellClass(col, row))) {
                    int index = getCellStyleIndex(col, row);
                    if (ret[index] == null) {
                        ret[index] = new GridCell();
                        if (fullMode) {
                            ret[index].init(this, col, row);
                        } else {
                            ret[index].init(this, 0, 0, getCellPropData(col, row));
                        }
                    }
                }
            }
        }
        return ret;
    }

    public byte[] getCellPropData(int col, int row) {
        Integer index = (Integer) this.cells.getElement(col, row);
        if (index != null) {
            return this.propList.get(index.intValue());
        }
        if (col == 0 || row == 0) {
            return GridConsts.DEF_HEADCELL_PROP;
        }
        return GridConsts.DEF_CELL_PROP;
    }

    public GridCell getCell(int col, int row) {
        this.cell.init(this, col, row);
        return this.cell;
    }

    public GridCell getCell(String pos) {
        CellPostion p = new CellPostion(pos);
        return getCell(p.getCol(), p.getRow());
    }

    public GridCell getCellEx(int col, int row) {
        GridCell result = new GridCell();
        result.init(this, col, row);
        return result;
    }

    public GridCell getCellEx(String pos) {
        CellPostion p = new CellPostion(pos);
        return getCellEx(p.getCol(), p.getRow());
    }

    public GridCellProperty getCellForChange(int col, int row) {
        if (!checkCol(col, false) || !checkRow(row, false)) {
            return null;
        }
        GridCellProperty result = new GridCellProperty();
        result.init(this, col, row);
        return result;
    }

    public GridCellProperty getCellForChange(String pos) {
        CellPostion p = new CellPostion(pos);
        return getCellForChange(p.getCol(), p.getRow());
    }

    public void setCell(int col, int row, GridCell value) {
        int index = this.propList.indexOf(value.getPropData());
        if (index < 0) {
            index = this.propList.add(value.getPropData());
        }
        boolean dynamicStore = (this.options & 4) != 0;
        Integer oldIndex = dynamicStore ? (Integer) this.cells.getElement(col, row) : null;
        this.cells.setElement(col, row, new Integer(index));
        if (dynamicStore && oldIndex != null && oldIndex.intValue() != index) {
            rebuildPropIndex(col, row, oldIndex.intValue());
        }
    }

    public void setCell(GridCell value) {
        setCell(value.getColNum(), value.getRowNum(), value);
    }

    public void setCell(int left, int top, int right, int bottom, GridCell value) {
        setCell(new CellField(left, top, right, bottom), value);
    }

    public void setCell(CellField field, GridCell value) {
        int index = this.propList.indexOf(value.getPropData());
        if (index == -1) {
            index = this.propList.add(value.getPropData());
        }
        Integer p = new Integer(index);
        for (int col = field.left; col <= field.right; col++) {
            for (int row = field.top; row <= field.bottom; row++) {
                CellField f = this.rects.getMergeRect(col, row);
                if (f == null || (f.left == col && f.top == row)) {
                    this.cells.setElement(col, row, p);
                }
            }
        }
    }

    public void setCell(CellField[] fields, GridCell value) {
        if (fields == null || fields.length == 0) {
            return;
        }
        int index = this.propList.indexOf(value.getPropData());
        if (index == -1) {
            index = this.propList.add(value.getPropData());
        }
        Integer p = new Integer(index);
        for (int i = 0; i < fields.length; i++) {
            for (int col = fields[i].left; col <= fields[i].right; col++) {
                for (int row = fields[i].top; row <= fields[i].bottom; row++) {
                    CellField f = this.rects.getMergeRect(col, row);
                    if (f == null || (f.left == col && f.top == row)) {
                        this.cells.setElement(col, row, p);
                    }
                }
            }
        }
    }

    public Object getObj(int col, int row) {
        return this.objects.getElement(col, row);
    }

    public void setObj(int col, int row, Object obj) {
        this.objects.setElement(col, row, obj);
    }

    public boolean getColAutoSize(int col) {
        return this.colAutoSize.get(col);
    }

    public void setColAutoSize(int col, boolean value) {
        this.colAutoSize.set(col, value);
    }

    public boolean getColVisible(int col) {
        if (this.colVisible.count() > 0) {
            return this.colVisible.get(col);
        }
        return true;
    }

    public void setColVisible(int col, boolean value) {
        if (!value || col < this.colVisible.count()) {
            this.colVisible.set(col, value);
        }
    }

    public int getColCount() {
        return this.colWidths.count();
    }

    public int getColWidths(int col) {
        return this.colWidths.get(col);
    }

    public void setColWidths(int col, int value) {
        this.colWidths.set(col, value);
    }

    public boolean getRowAutoSize(int row) {
        return this.rowAutoSize.get(row);
    }

    public void setRowAutoSize(int row, boolean value) {
        this.rowAutoSize.set(row, value);
    }

    public boolean getRowVisible(int row) {
        if (this.rowVisible.count() > 0) {
            return this.rowVisible.get(row);
        }
        return true;
    }

    public void setRowVisible(int row, boolean value) {
        if (!value || row < getRowCount()) {
            this.rowVisible.set(row, value);
        }
    }

    public boolean getHideDataRow(int row) {
        return this.hideDataRow.get(row);
    }

    public void setHideDataRow(int row, boolean value) {
        if (!value || row < getRowCount()) {
            this.hideDataRow.set(row, value);
        }
    }

    public boolean getHideDataCol(int col) {
        return this.hideDataCol.get(col);
    }

    public int[][] getHideDataCols() {
        int[][] result = (int[][]) null;
        int count = 0;
        boolean state = false;
        for (int i = 1; i < getColCount(); i++) {
            if (!state && getHideDataCol(i)) {
                count++;
                state = true;
            }
            if (state && !getHideDataCol(i)) {
                state = false;
            }
        }
        if (count > 0) {
            result = new int[2][count];
            boolean state2 = false;
            int k = 0;
            for (int i2 = 1; i2 < getColCount(); i2++) {
                if (!state2 && getHideDataCol(i2)) {
                    result[0][k] = i2;
                    state2 = true;
                }
                if (state2 && !getHideDataCol(i2)) {
                    state2 = false;
                    result[1][k] = i2 - 1;
                    k++;
                }
            }
            if (state2) {
                result[1][count - 1] = getColCount() - 1;
            }
        }
        return result;
    }

    public int[][] getHideDataRows() {
        int[][] result = (int[][]) null;
        int count = 0;
        boolean state = false;
        for (int i = 1; i < getRowCount(); i++) {
            if (!state && getHideDataRow(i)) {
                count++;
                state = true;
            }
            if (state && !getHideDataRow(i)) {
                state = false;
            }
        }
        if (count > 0) {
            result = new int[2][count];
            boolean state2 = false;
            int k = 0;
            for (int i2 = 1; i2 < getRowCount(); i2++) {
                if (!state2 && getHideDataRow(i2)) {
                    result[0][k] = i2;
                    state2 = true;
                }
                if (state2 && !getHideDataRow(i2)) {
                    state2 = false;
                    result[1][k] = i2 - 1;
                    k++;
                }
            }
            if (state2) {
                result[1][count - 1] = getRowCount() - 1;
            }
        }
        return result;
    }

    public void setHideDataCol(int col, boolean value) {
        if (!value || col < getColCount()) {
            this.hideDataCol.set(col, value);
        }
    }

    public int getRowCount() {
        return this.rowHeights.count();
    }

    public void setColCount(int value) {
        int colCount = getColCount();
        if (value > 0 && value < colCount) {
            int delCount = colCount - value;
            this.colVisible.delete(value, delCount);
            this.colAutoSize.delete(value, delCount);
            this.colWidths.delete(value, delCount);
            this.values.xDelete(value, delCount);
            this.cells.xDelete(value, delCount);
            this.objects.xDelete(value, delCount);
            if (this.cellCSS != null) {
                this.cellCSS.xDelete(value, delCount);
            }
            if (this.cellScript != null) {
                this.cellScript.xDelete(value, delCount);
            }
            this.rects.delCols(value, delCount);
        } else if (value > colCount) {
            int insCount = value - colCount;
            this.colVisible.insert(colCount, insCount);
            this.colAutoSize.insert(colCount, insCount);
            this.colWidths.insert(colCount, insCount, colCount == 0 ? 100 : this.colWidths.get(colCount - 1));
            if (colCount == 0) {
                this.colWidths.set(0, 50);
            }
            this.values.xInsert(colCount, insCount);
            this.cells.xInsert(colCount, insCount);
            this.objects.xInsert(colCount, insCount);
            if (this.cellCSS != null) {
                this.cellCSS.xInsert(colCount, insCount);
            }
            if (this.cellScript != null) {
                this.cellScript.xInsert(colCount, insCount);
            }
            this.rects.insertCols(colCount, insCount);
            if (getRowCount() > 0) {
                for (int i = colCount; i < colCount + insCount; i++) {
                    GridCell cell = getCell(i, 0);
                    setCell(i, 0, cell);
                }
            }
        }
        int colCount2 = getColCount();
        if (this.scrollTopCol >= colCount2) {
            setScrollTopCol(colCount2 - 1);
        }
        if (this.scrollBottomCol >= colCount2) {
            setScrollBottomCol(colCount2 - 1);
        }
    }

    public void setRowCount(int value) {
        int rowCount = getRowCount();
        if (value < rowCount) {
            int delCount = rowCount - value;
            this.rowVisible.delete(value, delCount);
            this.rowAutoSize.delete(value, delCount);
            this.rowHeights.delete(value, delCount);
            this.values.yDelete(value, delCount);
            this.cells.yDelete(value, delCount);
            this.objects.yDelete(value, delCount);
            if (this.cellCSS != null) {
                this.cellCSS.yDelete(value, delCount);
            }
            if (this.cellScript != null) {
                this.cellScript.yDelete(value, delCount);
            }
            this.rects.delRows(value, delCount);
            return;
        }
        if (value > getRowCount()) {
            int insCount = value - rowCount;
            this.rowVisible.insert(rowCount, insCount);
            this.rowAutoSize.insert(rowCount, insCount);
            this.rowHeights.insert(rowCount, insCount, rowCount == 0 ? 20 : this.rowHeights.get(rowCount - 1));
            this.values.yInsert(rowCount, insCount);
            this.objects.yInsert(rowCount, insCount);
            this.cells.yInsert(rowCount, insCount);
            if (this.cellCSS != null) {
                this.cellCSS.yInsert(rowCount, insCount);
            }
            if (this.cellScript != null) {
                this.cellScript.yInsert(rowCount, insCount);
            }
            this.rects.insertRows(rowCount, insCount);
            if (getColCount() > 0) {
                for (int i = rowCount; i < rowCount + insCount; i++) {
                    GridCell cell = getCell(0, i);
                    setCell(0, i, cell);
                }
            }
        }
    }

    public void insertCol(int index, int count) {
        insertCol(index, count, (this.options & 2) == 0, (this.options & 8) != 0);
    }

    private void insertCol(int index, int count, boolean fastMode, boolean fullMode) {
        GridCell cell;
        if (!checkCol(index, true)) {
            return;
        }
        this.colVisible.insert(index, count);
        if (index > 0) {
            this.colAutoSize.insert(index, count, this.colAutoSize.get(index - 1));
        } else {
            this.colAutoSize.insert(index, count);
        }
        this.colWidths.insert(index, count, (getColCount() == 0 || index == 1) ? 100 : this.colWidths.get(index - 1));
        this.values.xInsert(index, count);
        this.cells.xInsert(index, count);
        this.objects.xInsert(index, count);
        if (this.cellCSS != null) {
            this.cellCSS.xInsert(index, count);
        }
        if (this.cellScript != null) {
            this.cellScript.xInsert(index, count);
        }
        this.rects.insertCols(index, count);
        if (getRowCount() > 0 && !fastMode) {
            for (int row = 1; row < getRowCount(); row++) {
                if (index > 1) {
                    cell = getCell(index - 1, row);
                } else {
                    cell = this.cell;
                    cell.init(this, -1, -1, GridConsts.DEF_CELL_PROP);
                }
                for (int col = index; col < index + count; col++) {
                    setCell(col, row, cell);
                    if (fullMode) {
                        setCellClass(col, row, cell.getCssClass());
                        setCellScript(col, row, cell.getScript());
                    }
                }
            }
        }
        if (this.scrollTopCol >= index) {
            this.scrollTopCol += count;
        }
        if (this.scrollBottomCol + 1 >= index) {
            this.scrollBottomCol += count;
        }
    }

    public void deleteCol(int index, int count) {
        if (!checkCol(index, false)) {
            return;
        }
        if (index + count > getColCount()) {
            count = getColCount() - index;
        }
        this.colVisible.delete(index, count);
        this.colAutoSize.delete(index, count);
        this.colWidths.delete(index, count);
        this.values.xDelete(index, count);
        this.cells.xDelete(index, count);
        this.objects.xDelete(index, count);
        if (this.cellCSS != null) {
            this.cellCSS.xDelete(index, count);
        }
        if (this.cellScript != null) {
            this.cellScript.xDelete(index, count);
        }
        this.rects.delCols(index, count);
        if (this.scrollTopCol >= index) {
            this.scrollTopCol = Math.min(Math.max(index, this.scrollTopCol - count), getColCount() - 1);
        }
        if (this.scrollBottomCol >= index) {
            this.scrollBottomCol = Math.min(Math.max(index, this.scrollBottomCol - count), getColCount() - 1);
        }
    }

    public void insertRow(int index, int count) {
        insertRow(index, count, (this.options & 2) == 0, (this.options & 8) != 0);
    }

    private void insertRow(int index, int count, boolean fastMode, boolean fullMode) {
        GridCell cell;
        if (!checkRow(index, true)) {
            return;
        }
        this.rowVisible.insert(index, count);
        if (index > 0) {
            this.rowAutoSize.insert(index, count, this.rowAutoSize.get(index - 1));
        } else {
            this.rowAutoSize.insert(index, count);
        }
        this.rowHeights.insert(index, count, (getRowCount() == 0 || index == 1) ? 20 : this.rowHeights.get(index - 1));
        this.values.yInsert(index, count);
        this.cells.yInsert(index, count);
        this.objects.yInsert(index, count);
        if (this.cellCSS != null) {
            this.cellCSS.yInsert(index, count);
        }
        if (this.cellScript != null) {
            this.cellScript.yInsert(index, count);
        }
        this.rects.insertRows(index, count);
        if (getColCount() > 0 && !fastMode) {
            for (int col = 1; col < getColCount(); col++) {
                if (index > 1) {
                    cell = getCell(col, index - 1);
                } else {
                    cell = this.cell;
                    cell.init(this, -1, -1, GridConsts.DEF_CELL_PROP);
                }
                for (int row = index; row < index + count; row++) {
                    setCell(col, row, cell);
                    if (fullMode) {
                        setCellClass(col, row, cell.getCssClass());
                        setCellScript(col, row, cell.getScript());
                    }
                }
            }
        }
        if (this.scrollTopRow >= index) {
            this.scrollTopRow += count;
        }
        if (this.scrollBottomRow + 1 >= index) {
            this.scrollBottomRow += count;
        }
    }

    public boolean duplicateCols(int fromIndex, int toIndex, int duplicateCount, int insertIndex) {
        if (!checkCol(fromIndex, false) || !checkCol(toIndex, false) || !checkCol(insertIndex, true) || toIndex < fromIndex || duplicateCount <= 0) {
            return false;
        }
        if (insertIndex > fromIndex && insertIndex <= toIndex) {
            return false;
        }
        int insertCount = ((toIndex - fromIndex) + 1) * duplicateCount;
        insertCol(insertIndex, insertCount, true, true);
        if (insertIndex <= fromIndex) {
            fromIndex += insertCount;
            toIndex += insertCount;
        }
        List merges = fromIndex < toIndex ? this.rects.findFieldsInCols(fromIndex, toIndex) : null;
        for (int i = 0; i < duplicateCount; i++) {
            int offset = (insertIndex + (((toIndex - fromIndex) + 1) * i)) - fromIndex;
            for (int j = fromIndex; j <= toIndex; j++) {
                this.colWidths.set(offset + j, this.colWidths.get(j));
                this.values.xCopy(j, offset + j);
                if (this.cellCSS != null) {
                    this.cellCSS.xCopy(j, offset + j);
                }
                if (this.cellScript != null) {
                    this.cellScript.xCopy(j, offset + j);
                }
                this.cells.xCopy(j, offset + j);
                if (!this.colVisible.get(j)) {
                    this.colVisible.set(j + offset, false);
                }
            }
            if (merges != null && !merges.isEmpty()) {
                for (int j2 = 0; j2 < merges.size(); j2++) {
                    CellField cf = (CellField) merges.get(j2);
                    mergeCells(cf.left + offset, cf.top, cf.right + offset, cf.bottom);
                }
            }
        }
        return true;
    }

    public boolean duplicateRows(int fromIndex, int toIndex, int duplicateCount, int insertIndex) {
        if (!checkRow(fromIndex, false) || !checkRow(toIndex, false) || !checkRow(insertIndex, true) || toIndex < fromIndex || duplicateCount <= 0) {
            return false;
        }
        if (insertIndex > fromIndex && insertIndex <= toIndex) {
            return false;
        }
        int insertCount = ((toIndex - fromIndex) + 1) * duplicateCount;
        insertRow(insertIndex, insertCount, true, true);
        if (insertIndex <= fromIndex) {
            fromIndex += insertCount;
            toIndex += insertCount;
        }
        List merges = fromIndex < toIndex ? this.rects.findFieldsInRows(fromIndex, toIndex) : null;
        for (int i = 0; i < duplicateCount; i++) {
            int offset = (insertIndex + (((toIndex - fromIndex) + 1) * i)) - fromIndex;
            for (int j = fromIndex; j <= toIndex; j++) {
                this.rowHeights.set(offset + j, this.rowHeights.get(j));
                this.values.yCopy(j, offset + j);
                if (this.cellCSS != null) {
                    this.cellCSS.yCopy(j, offset + j);
                }
                if (this.cellScript != null) {
                    this.cellScript.yCopy(j, offset + j);
                }
                this.cells.yCopy(j, offset + j);
                if (!this.rowVisible.get(j)) {
                    this.rowVisible.set(j + offset, false);
                }
            }
            if (merges != null && !merges.isEmpty()) {
                for (int j2 = 0; j2 < merges.size(); j2++) {
                    CellField cf = (CellField) merges.get(j2);
                    mergeCells(cf.left, cf.top + offset, cf.right, cf.bottom + offset);
                }
            }
        }
        return true;
    }

    public void deleteRow(int index, int count) {
        if (!checkRow(index, false)) {
            return;
        }
        if (index + count > getRowCount()) {
            count = getRowCount() - index;
        }
        this.rowVisible.delete(index, count);
        this.rowAutoSize.delete(index, count);
        this.rowHeights.delete(index, count);
        this.values.yDelete(index, count);
        this.cells.yDelete(index, count);
        if (this.cellCSS != null) {
            this.cellCSS.yDelete(index, count);
        }
        if (this.cellScript != null) {
            this.cellScript.yDelete(index, count);
        }
        this.objects.yDelete(index, count);
        this.rects.delRows(index, count);
        if (this.scrollTopRow >= index) {
            this.scrollTopRow = Math.min(Math.max(index, this.scrollTopRow - count), getRowCount() - 1);
        }
        if (this.scrollBottomRow >= index) {
            this.scrollBottomRow = Math.min(Math.max(index, this.scrollBottomRow - count), getRowCount() - 1);
        }
    }

    public int getRowHeights(int row) {
        return this.rowHeights.get(row);
    }

    public void setRowHeights(int row, int value) {
        this.rowHeights.set(row, value);
    }

    public int getScrollTopCol() {
        return this.scrollTopCol;
    }

    public void setScrollTopCol(int value) {
        this.scrollTopCol = value;
    }

    public int getScrollTopRow() {
        return this.scrollTopRow;
    }

    public void setScrollTopRow(int value) {
        this.scrollTopRow = value;
    }

    public int getScrollBottomCol() {
        return this.scrollBottomCol;
    }

    public void setScrollBottomCol(int value) {
        this.scrollBottomCol = value;
    }

    public int getScrollBottomRow() {
        return this.scrollBottomRow;
    }

    public void setScrollBottomRow(int value) {
        this.scrollBottomRow = value;
    }

    public int lockTopCol() {
        return this.scrollTopCol;
    }

    public void setLockTopCol(int value) {
        this.scrollTopCol = value;
    }

    public int lockTopRow() {
        return this.scrollTopRow;
    }

    public void setLockTopRow(int value) {
        this.scrollTopRow = value;
    }

    public int lockBottomCol() {
        return this.scrollBottomCol;
    }

    public void setLockBottomCol(int value) {
        this.scrollBottomCol = value;
    }

    public int lockBottomRow() {
        return this.scrollBottomRow;
    }

    public void setLockBottomRow(int value) {
        this.scrollBottomRow = value;
    }

    public int curCol() {
        return this.curCol;
    }

    public void setCurCol(int value) {
        this.curCol = value;
    }

    public int curRow() {
        return this.curRow;
    }

    public void setCurRow(int value) {
        this.curRow = value;
    }

    public GridFieldList merges() {
        return this.rects;
    }

    public IntList cellColors() {
        return this.cellColors;
    }

    public IntList fontColors() {
        return this.fontColors;
    }

    public IntList edgeColors() {
        return this.edgeColors;
    }

    public void setEdgeColor(int index, int color) {
        if (index >= 0 && index < 16) {
            this.edgeColors.set(index, color);
        }
    }

    public GridFontNames fontNames() {
        return this.fontNames;
    }

    public IntList fontSizes() {
        return this.fontSizes;
    }

    public boolean cellMerged(int col, int row) {
        return this.rects.getMergeRect(col, row) != null;
    }

    public CellField expandCell(int col, int row) {
        CellField rt = this.rects.getMergeRect(col, row);
        return rt == null ? new CellField(col, row, col, row) : (CellField) rt.clone();
    }

    public boolean mergeCells(int left, int top, int right, int bottom) {
        CellField cf = new CellField(left, top, right, bottom);
        return this.rects.addMergeRect(cf);
    }

    public LabelItems getLabels() {
        return this.labels;
    }

    private void clear() {
        this.scrollTopCol = -1;
        this.scrollTopRow = -1;
        this.scrollBottomCol = -1;
        this.scrollBottomRow = -1;
        this.curCol = -1;
        this.curRow = -1;
        this.labels.clear();
        this.extDatas.clear();
        this.cellCSS = null;
        this.cellScript = null;
    }

    public void loadFromStream(Stream stream) throws StreamException {
        int fileVer;
        ValueList valueList = new ValueList();
        ValueList cellCSSList = new ValueList();
        ValueList cellScriptList = new ValueList();
        WordList propIndexs = new WordList();
        KeyValues dataProps = new KeyValues();
        BIFF b = new BIFF();
        b.data().setUseEncode(stream.getUseEncode());
        b.data().setCharset(stream.getCharset());
        long p = stream.getPosition();
        BIFF.readBIFF(1, b, stream);
        String signFlag = b.data().readString(4);
        if (!GridConsts.BIFF_DATA_FLAG.equals(signFlag)) {
            stream.setPosition(p);
            BIFF.readBIFF(2, b, stream);
            String signFlag2 = b.data().readString(4);
            if (!GridConsts.BIFF_DATA_FLAG.equals(signFlag2)) {
                throw new StreamException("错误的文件格式");
            }
        }
        int ver = b.data().readInt();
        if (b.data().getPosition() < b.data().getSize()) {
            fileVer = b.data().readInt();
        } else {
            fileVer = 16777216;
        }
        clear();
        this.useFontSize = false;
        while (b.ident != 127) {
            BIFF.readBIFF(ver, b, stream);
            switch (b.ident) {
                case 1:
                    propIndexs.loadFromStream(b.data(), b.size);
                    break;
                case 2:
                    this.colWidths.loadFromStream(b.data(), b.size);
                    break;
                case 3:
                    this.rowHeights.loadFromStream(b.data(), b.size);
                    break;
                case 4:
                    valueList.loadFromStream(b.data());
                    break;
                case 5:
                    this.rects.loadFromStream(b.data(), b.size);
                    break;
                case 6:
                    this.fontNames.loadFromStream(b.data());
                    break;
                case 7:
                    this.fontColors.loadFromStream(b.data(), b.size);
                    break;
                case 8:
                    this.fontSizes.loadFromStream(b.data(), b.size);
                    break;
                case 9:
                    this.edgeColors.loadFromStream(b.data(), b.size);
                    break;
                case 10:
                    this.cellColors.loadFromStream(b.data(), b.size);
                    break;
                case 11:
                    this.propList.loadFromStreamEx(b.data(), 0, 12);
                    break;
                case 12:
                    this.colVisible.loadFromStream(b.data(), b.size);
                    break;
                case 13:
                    this.colAutoSize.loadFromStream(b.data(), b.size);
                    break;
                case 14:
                    this.rowVisible.loadFromStream(b.data(), b.size);
                    break;
                case 15:
                    this.rowAutoSize.loadFromStream(b.data(), b.size);
                    break;
                case 16:
                    this.scrollTopCol = b.data().readInt();
                    this.scrollTopRow = b.data().readInt();
                    this.scrollBottomCol = b.data().readInt();
                    this.scrollBottomRow = b.data().readInt();
                    break;
                case 17:
                    this.curCol = b.data().readInt();
                    this.curRow = b.data().readInt();
                    break;
                case 18:
                    this.useFontSize = true;
                    break;
                case 19:
                    this.propList.loadFromStreamEx(b.data(), 12, 8);
                    break;
                case 20:
                    this.labels.loadFromStream(b.data());
                    if (fileVer == 16777216) {
                        this.labels.ver10Tover11();
                        break;
                    } else {
                        break;
                    }
                case 21:
                    this.hideDataCol.loadFromStream(b.data(), b.size);
                    break;
                case 22:
                    this.hideDataRow.loadFromStream(b.data(), b.size);
                    break;
                case 23:
                    this.colCountPerPage = b.data().readInt();
                    this.rowCountPerPage = b.data().readInt();
                    this.rowPagingFirst = b.data().readBool();
                    break;
                case 24:
                    cellCSSList.loadFromStream(b.data());
                    break;
                case 25:
                    dataProps.loadFromStream(b.data());
                    break;
                case 26:
                    cellScriptList.loadFromStream(b.data());
                    break;
                case 27:
                    this.extDatas.loadFromStream(b.data(), b.size);
                    break;
            }
        }
        this.gridDataClass = dataProps.getValue(GridConsts.KEY_GridDataClass);
        this.referenceBase = dataProps.getValue(GridConsts.KEY_GridDataRefBase);
        this.horzOffset = dataProps.getAsInt(GridConsts.KEY_HorzOffset);
        this.vertOffset = dataProps.getAsInt(GridConsts.KEY_VertOffset);
        this.horzCenter = dataProps.getAsBoolean(GridConsts.KEY_HorzCenter);
        this.vertCenter = dataProps.getAsBoolean(GridConsts.KEY_VertCenter);
        setColCount(this.colWidths.count());
        setRowCount(this.rowHeights.count());
        this.cells.resetSize(this.colWidths.count(), this.rowHeights.count());
        this.objects.resetSize(this.colWidths.count(), this.rowHeights.count());
        this.values.resetSize(this.colWidths.count(), this.rowHeights.count());
        int i = 0;
        for (int c = 0; c < this.colWidths.count(); c++) {
            for (int r = 0; r < this.rowHeights.count(); r++) {
                this.cells.setElement(c, r, new Integer(propIndexs.get(i)));
                i++;
            }
        }
        for (int i2 = 0; i2 < valueList.count(); i2++) {
            this.values.setElement(valueList.col(i2), valueList.row(i2), valueList.get(i2));
        }
        for (int i3 = 0; i3 < cellCSSList.count(); i3++) {
            setCellClass(cellCSSList.col(i3), cellCSSList.row(i3), cellCSSList.get(i3));
        }
        for (int i4 = 0; i4 < cellScriptList.count(); i4++) {
            setCellScript(cellScriptList.col(i4), cellScriptList.row(i4), cellScriptList.get(i4));
        }
    }

    public void saveToStream(Stream stream) throws StreamException {
        int pi;
        ValueList valueList = new ValueList();
        ValueList cellCSSList = new ValueList();
        ValueList cellScriptList = new ValueList();
        WordList propIndexs = new WordList();
        KeyValues dataProps = new KeyValues();
        dataProps.setValue(GridConsts.KEY_GridDataClass, this.gridDataClass);
        dataProps.setValue(GridConsts.KEY_GridDataRefBase, this.referenceBase);
        dataProps.setAsInt(GridConsts.KEY_HorzOffset, this.horzOffset);
        dataProps.setAsInt(GridConsts.KEY_VertOffset, this.vertOffset);
        dataProps.setAsBoolean(GridConsts.KEY_HorzCenter, this.horzCenter);
        dataProps.setAsBoolean(GridConsts.KEY_VertCenter, this.vertCenter);
        int c = 0;
        while (c < this.colWidths.count()) {
            int r = 0;
            while (r < this.rowHeights.count()) {
                String cell = (String) this.values.getElement(c, r);
                if (cell != null && cell.length() > 0) {
                    CellValue cv = new CellValue();
                    cv.col = c;
                    cv.row = r;
                    cv.value = cell;
                    valueList.add(cv);
                }
                Integer propIndex = (Integer) this.cells.getElement(c, r);
                if (propIndex == null) {
                    this.propList.indexOf((c == 0 || r == 0) ? GridConsts.DEF_HEADCELL_PROP : GridConsts.DEF_CELL_PROP);
                    if (c == 0 || r == 0) {
                        pi = this.propList.add(GridConsts.DEF_HEADCELL_PROP);
                    } else {
                        pi = this.propList.add(GridConsts.DEF_CELL_PROP);
                    }
                    propIndex = new Integer(pi);
                }
                propIndexs.add(propIndex);
                String cell2 = getCellClass(c, r);
                if (cell2 != null && cell2.length() > 0) {
                    CellValue cv2 = new CellValue();
                    cv2.col = c;
                    cv2.row = r;
                    cv2.value = cell2;
                    cellCSSList.add(cv2);
                }
                String cell3 = getCellScript(c, r);
                if (cell3 != null && cell3.length() != 0) {
                    CellValue cv3 = new CellValue();
                    cv3.col = c;
                    cv3.row = r;
                    cv3.value = cell3;
                    cellScriptList.add(cv3);
                }
                r++;
            }
            c++;
        }
        BIFF b = new BIFF();
        b.ident = (short) 0;
        b.data().writeString(GridConsts.BIFF_DATA_FLAG);
        b.data().writeInt(2);
        b.data().writeInt(16842752);
        BIFF.writeBIFF(1, b, stream);
        b.reset();
        b.ident = (short) 25;
        dataProps.saveToStream(b.data());
        BIFF.writeBIFF(25, b, stream);
        b.reset();
        b.ident = (short) 18;
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 6;
        this.fontNames.saveToStream(b.data());
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 8;
        this.fontSizes.saveToStream(b.data());
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 7;
        this.fontColors.saveToStream(b.data());
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 10;
        this.cellColors.saveToStream(b.data());
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 9;
        this.edgeColors.saveToStream(b.data());
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 2;
        this.colWidths.saveToStream(b.data());
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 13;
        this.colAutoSize.saveToStream(b.data());
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 12;
        this.colVisible.saveToStream(b.data());
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 21;
        this.hideDataCol.saveToStream(b.data());
        BIFF.writeBIFF(21, b, stream);
        b.reset();
        b.ident = (short) 3;
        this.rowHeights.saveToStream(b.data());
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 15;
        this.rowAutoSize.saveToStream(b.data());
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 14;
        this.rowVisible.saveToStream(b.data());
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 22;
        this.hideDataRow.saveToStream(b.data());
        BIFF.writeBIFF(22, b, stream);
        b.reset();
        b.ident = (short) 4;
        valueList.saveToStream(b.data());
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 11;
        this.propList.saveToStreamEx(b.data(), 0, 12);
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 19;
        this.propList.saveToStreamEx(b.data(), 12, 8);
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 1;
        propIndexs.saveToStream(b.data());
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 5;
        this.rects.saveToStream(b.data());
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 16;
        b.data().writeInt(this.scrollTopCol);
        b.data().writeInt(this.scrollTopRow);
        b.data().writeInt(this.scrollBottomCol);
        b.data().writeInt(this.scrollBottomRow);
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 17;
        b.data().writeInt(this.curCol);
        b.data().writeInt(this.curRow);
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 20;
        this.labels.saveToStream(b.data());
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 23;
        b.data().writeInt(this.colCountPerPage);
        b.data().writeInt(this.rowCountPerPage);
        b.data().writeBool(this.rowPagingFirst);
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 24;
        cellCSSList.saveToStream(b.data());
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 26;
        cellScriptList.saveToStream(b.data());
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 27;
        this.extDatas.saveToStream(b.data());
        BIFF.writeBIFF(2, b, stream);
        b.reset();
        b.ident = (short) 127;
        BIFF.writeBIFF(2, b, stream);
    }

    public void loadFromStream(InputStream inStream) throws StreamException {
        MemStream stream = new MemStream();
        try {
            stream.loadFromStream(inStream);
            stream.setPosition(0L);
            loadFromStream((Stream) stream);
        } catch (IOException e) {
            throw new StreamException(e);
        }
    }

    public void saveToStream(OutputStream outStream) throws StreamException {
        MemStream stream = new MemStream();
        saveToStream((Stream) stream);
        try {
            stream.saveToStream(outStream);
        } catch (IOException e) {
            throw new StreamException(e);
        }
    }

    public void loadFromFile(String fileName) throws StreamException {
        MemStream stream = new MemStream();
        try {
            stream.loadFromFile(fileName);
            stream.setPosition(0L);
            loadFromStream((Stream) stream);
        } catch (IOException e) {
            throw new StreamException(e);
        }
    }

    public void saveToFile(String fileName) throws StreamException {
        MemStream stream = new MemStream();
        saveToStream((Stream) stream);
        try {
            stream.saveToFile(fileName);
        } catch (IOException e) {
            throw new StreamException(e);
        }
    }

    public GridData makeCopy() throws StreamException {
        Stream s = new MemStream();
        saveToStream(s);
        s.setPosition(0L);
        GridData gd = new GridData();
        gd.loadFromStream(s);
        gd.setOptions(getOptions());
        return gd;
    }

    public Object clone() {
        try {
            return makeCopy();
        } catch (StreamException unused) {
            return null;
        }
    }

    public boolean copyFrom(GridData srcData, int fromLeft, int fromTop, int fromRight, int fromBottom, int toLeft, int toTop) {
        CellField dest = new CellField(toLeft, toTop, (toLeft + fromRight) - fromLeft, (toTop + fromBottom) - fromTop);
        if (!checkCol(dest.left, false) || !checkCol(dest.right, false) || !checkRow(dest.top, false) || !checkRow(dest.bottom, false) || !srcData.checkCol(fromLeft, false) || !srcData.checkCol(fromRight, false) || !srcData.checkRow(fromTop, false) || !srcData.checkRow(fromBottom, false)) {
            return false;
        }
        this.rects.removeMergeRect(dest);
        GridCopyConvertor convertor = this == srcData ? null : new GridCopyConvertor(srcData);
        for (int x = fromLeft; x <= fromRight; x++) {
            for (int y = fromTop; y <= fromBottom; y++) {
                CellField field = srcData.expandCell(x, y);
                String text = srcData.getCellData(field.left, field.top);
                GridCell cell = srcData.getCellEx(field.left, field.top);
                if (field.left < fromLeft) {
                    field.left = fromLeft;
                }
                if (field.right > fromRight) {
                    field.right = fromRight;
                }
                if (field.top < fromTop) {
                    field.top = fromTop;
                }
                if (field.bottom > fromBottom) {
                    field.bottom = fromBottom;
                }
                if (field.left >= x && field.top >= y) {
                    int xx = (toLeft + x) - fromLeft;
                    int yy = (toTop + y) - fromTop;
                    if (convertor != null) {
                        convertor.convert(cell);
                    }
                    setCell(xx, yy, cell);
                    setCellData(xx, yy, text);
                    setObj(xx, yy, srcData.getObj(field.left, field.top));
                    setCellClass(xx, yy, srcData.getCellClass(field.left, field.top));
                    setCellScript(xx, yy, srcData.getCellScript(field.left, field.top));
                    mergeCells(xx, yy, (toLeft + field.right) - fromLeft, (toTop + field.bottom) - fromTop);
                }
            }
        }
        if (dest.top == 1 && dest.bottom == getRowCount() - 1) {
            for (int x2 = fromLeft; x2 <= fromRight; x2++) {
                int xx2 = (toLeft + x2) - fromLeft;
                setHideDataCol(xx2, srcData.getHideDataCol(x2));
                setColAutoSize(xx2, srcData.getColAutoSize(x2));
                setColWidths(xx2, srcData.getColWidths(x2));
                setColVisible(xx2, srcData.getColVisible(x2));
            }
        }
        if (dest.left == 1 && dest.right == getColCount() - 1) {
            for (int y2 = fromTop; y2 <= fromBottom; y2++) {
                int yy2 = (toTop + y2) - fromTop;
                setHideDataRow(yy2, srcData.getHideDataRow(y2));
                setRowAutoSize(yy2, srcData.getRowAutoSize(y2));
                setRowHeights(yy2, srcData.getRowHeights(y2));
                setRowVisible(yy2, srcData.getRowVisible(y2));
            }
            return true;
        }
        return true;
    }

    public int getColCountPerPage() {
        return this.colCountPerPage;
    }

    public void setColCountPerPage(int value) {
        this.colCountPerPage = value;
    }

    public int getRowCountPerPage() {
        return this.rowCountPerPage;
    }

    public void setRowCountPerPage(int value) {
        this.rowCountPerPage = value;
    }

    public boolean getRowPagingFirst() {
        return this.rowPagingFirst;
    }

    public void setRowPagingFirst(boolean value) {
        this.rowPagingFirst = value;
    }

    public int getColPageCount() {
        if (this.colCountPerPage <= 0) {
            return 1;
        }
        int colCount = getColCount() - 1;
        if (this.scrollBottomCol > 0 && this.scrollBottomCol > this.scrollTopCol) {
            colCount = this.scrollBottomCol;
        }
        if (this.scrollTopCol > 0) {
            colCount -= this.scrollTopCol - 1;
        }
        return ((colCount - 1) / this.colCountPerPage) + 1;
    }

    public int getRowPageCount() {
        if (this.rowCountPerPage <= 0) {
            return 1;
        }
        int rowCount = getRowCount() - 1;
        if (this.scrollBottomRow > 0 && this.scrollBottomRow > this.scrollTopRow) {
            rowCount = this.scrollBottomRow;
        }
        if (this.scrollTopRow > 0) {
            rowCount -= this.scrollTopRow - 1;
        }
        return ((rowCount - 1) / this.rowCountPerPage) + 1;
    }

    public int getPageCount() {
        return getColPageCount() * getRowPageCount();
    }

    public GridData getPagedGridData(int pageIndex) {
        int px;
        int py;
        if (this.rowPagingFirst) {
            int count = getRowPageCount();
            px = pageIndex / count;
            py = pageIndex % count;
        } else {
            int count2 = getColPageCount();
            px = pageIndex % count2;
            py = pageIndex / count2;
        }
        return getPagedGridData(px, py);
    }

    public GridData getPagedGridData(int colPageIndex, int rowPageIndex) {
        if (colPageIndex < 0 || rowPageIndex < 0) {
            return null;
        }
        int colHead = this.scrollTopCol - 1;
        int colTail = this.scrollBottomCol + 1;
        int rowHead = this.scrollTopRow - 1;
        int rowTail = this.scrollBottomRow + 1;
        if (colHead < 0) {
            colHead = 0;
        }
        if (colTail <= colHead) {
            colTail = getColCount();
        }
        if (rowHead < 0) {
            rowHead = 0;
        }
        if (rowTail <= rowHead) {
            rowTail = getRowCount();
        }
        int colCountPerPage = this.colCountPerPage;
        if (colCountPerPage <= 0) {
            colCountPerPage = (colTail - colHead) - 1;
        }
        int rowCountPerPage = this.rowCountPerPage;
        if (rowCountPerPage <= 0) {
            rowCountPerPage = (rowTail - rowHead) - 1;
        }
        int dataColCount = Math.min(colCountPerPage, ((colTail - (colPageIndex * colCountPerPage)) - colHead) - 1);
        if (dataColCount <= 0 && colPageIndex != 0) {
            return null;
        }
        int fromDataCol = colHead + 1 + (colPageIndex * colCountPerPage);
        int dataRowCount = Math.min(rowCountPerPage, ((rowTail - (rowPageIndex * rowCountPerPage)) - rowHead) - 1);
        if (dataRowCount <= 0 && rowPageIndex != 0) {
            return null;
        }
        int fromDataRow = rowHead + 1 + (rowPageIndex * rowCountPerPage);
        if (1 + colHead + dataColCount == colTail && 1 + rowHead + dataRowCount == rowTail) {
            return this;
        }
        GridData result = new GridData(getOptions());
        result.cellColors = (IntList) this.cellColors.clone();
        result.edgeColors = (IntList) this.edgeColors.clone();
        result.fontNames = (GridFontNames) this.fontNames.clone();
        result.fontColors = (IntList) this.fontColors.clone();
        result.fontSizes = (IntList) this.fontSizes.clone();
        result.setDataClass(getDataClass());
        result.setColCount((((1 + colHead) + dataColCount) + getColCount()) - colTail);
        result.setRowCount((((1 + rowHead) + dataRowCount) + getRowCount()) - rowTail);
        result.setLockTopCol(colHead + 1);
        result.setLockTopRow(rowHead + 1);
        result.setLockBottomCol(colHead + dataColCount);
        result.setLockBottomRow(rowHead + dataRowCount);
        for (int i = 1; i <= colHead; i++) {
            result.setColWidths(i, getColWidths(i));
            result.setColAutoSize(i, getColAutoSize(i));
            result.setColVisible(i, getColVisible(i));
            result.setHideDataCol(i, getHideDataCol(i));
        }
        for (int i2 = 0; i2 < dataColCount; i2++) {
            result.setColWidths(colHead + 1 + i2, getColWidths(fromDataCol + i2));
            result.setColAutoSize(colHead + 1 + i2, getColAutoSize(fromDataCol + i2));
            result.setColVisible(colHead + 1 + i2, getColVisible(fromDataCol + i2));
            result.setHideDataCol(colHead + 1 + i2, getHideDataCol(fromDataCol + i2));
        }
        for (int i3 = colTail; i3 < getColCount(); i3++) {
            result.setColWidths((((colHead + 1) + dataColCount) + i3) - colTail, getColWidths(i3));
            result.setColAutoSize((((colHead + 1) + dataColCount) + i3) - colTail, getColAutoSize(i3));
            result.setColVisible((((colHead + 1) + dataColCount) + i3) - colTail, getColVisible(i3));
            result.setHideDataCol((((colHead + 1) + dataColCount) + i3) - colTail, getHideDataCol(i3));
        }
        for (int i4 = 1; i4 <= rowHead; i4++) {
            result.setRowHeights(i4, getRowHeights(i4));
            result.setRowAutoSize(i4, getRowAutoSize(i4));
            result.setRowVisible(i4, getRowVisible(i4));
            result.setHideDataRow(i4, getHideDataRow(i4));
        }
        for (int i5 = 0; i5 < dataRowCount; i5++) {
            result.setRowHeights(rowHead + 1 + i5, getRowHeights(fromDataRow + i5));
            result.setRowAutoSize(rowHead + 1 + i5, getRowAutoSize(fromDataRow + i5));
            result.setRowVisible(rowHead + 1 + i5, getRowVisible(fromDataRow + i5));
            result.setHideDataRow(rowHead + 1 + i5, getHideDataRow(fromDataRow + i5));
        }
        for (int i6 = rowTail; i6 < getRowCount(); i6++) {
            result.setRowHeights((((rowHead + 1) + dataRowCount) + i6) - rowTail, getRowHeights(i6));
            result.setRowAutoSize((((rowHead + 1) + dataRowCount) + i6) - rowTail, getRowAutoSize(i6));
            result.setRowVisible((((rowHead + 1) + dataRowCount) + i6) - rowTail, getRowVisible(i6));
            result.setHideDataRow((((rowHead + 1) + dataRowCount) + i6) - rowTail, getHideDataRow(i6));
        }
        if (colHead > 0 && rowHead > 0) {
            result.copyFrom(this, 1, 1, colHead, rowHead, 1, 1);
        }
        if (dataColCount > 0 && rowHead > 0) {
            result.copyFrom(this, fromDataCol, 1, (fromDataCol + dataColCount) - 1, rowHead, colHead + 1, 1);
        }
        if (colTail < getColCount() && rowHead > 0) {
            result.copyFrom(this, colTail, 1, getColCount() - 1, rowHead, colHead + 1 + dataColCount, 1);
        }
        if (colHead > 0 && dataRowCount > 0) {
            result.copyFrom(this, 1, fromDataRow, colHead, (fromDataRow + dataRowCount) - 1, 1, rowHead + 1);
        }
        if (dataColCount > 0 && dataRowCount > 0) {
            result.copyFrom(this, fromDataCol, fromDataRow, (fromDataCol + dataColCount) - 1, (fromDataRow + dataRowCount) - 1, colHead + 1, rowHead + 1);
        }
        if (colTail < getColCount() && dataRowCount > 0) {
            result.copyFrom(this, colTail, fromDataRow, getColCount() - 1, (fromDataRow + dataRowCount) - 1, colHead + 1 + dataColCount, rowHead + 1);
        }
        if (colHead > 0 && rowTail < getRowCount()) {
            result.copyFrom(this, 1, rowTail, colHead, getRowCount() - 1, 1, rowHead + 1 + dataRowCount);
        }
        if (dataColCount > 0 && rowTail < getRowCount()) {
            result.copyFrom(this, fromDataCol, rowTail, (fromDataCol + dataColCount) - 1, getRowCount() - 1, colHead + 1, rowHead + 1 + dataRowCount);
        }
        if (colTail < getColCount() && rowTail < getRowCount()) {
            result.copyFrom(this, colTail, rowTail, getColCount() - 1, getRowCount() - 1, colHead + 1 + dataColCount, rowHead + 1 + dataRowCount);
        }
        return result;
    }

    public int getDataRowCount() {
        int top = this.scrollTopRow > 0 ? this.scrollTopRow : 1;
        int bottom = this.scrollBottomRow > 0 ? this.scrollBottomRow : getRowCount() - 1;
        int count = 0;
        for (int i = top; i <= bottom; i++) {
            if (getRowVisible(i)) {
                count++;
            }
        }
        return count;
    }

    public int getDataColCount() {
        int left = this.scrollTopCol > 0 ? this.scrollTopCol : 1;
        int right = this.scrollBottomCol > 0 ? this.scrollBottomCol : getColCount() - 1;
        int count = 0;
        for (int i = left; i <= right; i++) {
            if (getColVisible(i)) {
                count++;
            }
        }
        return count;
    }

    public ExtendedDatas getExtDatas() {
        return this.extDatas;
    }

    public int getHorzOffset() {
        return this.horzOffset;
    }

    public void setHorzOffset(int offset) {
        this.horzOffset = offset;
    }

    public int getVertOffset() {
        return this.vertOffset;
    }

    public void setVertOffset(int offset) {
        this.vertOffset = offset;
    }

    public boolean getHorzCenter() {
        return this.horzCenter;
    }

    public void setHorzCenter(boolean value) {
        this.horzCenter = value;
    }

    public boolean getVertCenter() {
        return this.vertCenter;
    }

    public void setVertCenter(boolean value) {
        this.vertCenter = value;
    }

    public static GridData bytesToGrid(byte[] data) {
        if (data == null) {
            return null;
        }
        GridData gd = new GridData();
        try {
            MemStream memStream = new MemStream();
            memStream.writeBuffer(data, 0, data.length);
            memStream.setPosition(0L);
            gd.loadFromStream((Stream) memStream);
        } catch (Exception ex) {
            ex.printStackTrace();
            gd = null;
        }
        return gd;
    }

    public static GridData bytesToGrid(byte[] data, String encodeCharset) {
        if (data == null) {
            return null;
        }
        GridData gd = new GridData();
        try {
            MemStream memStream = new MemStream();
            memStream.setUseEncode(true);
            memStream.setCharset(encodeCharset);
            memStream.writeBuffer(data, 0, data.length);
            memStream.setPosition(0L);
            gd.loadFromStream((Stream) memStream);
        } catch (Exception ex) {
            ex.printStackTrace();
            gd = null;
        }
        return gd;
    }

    public static byte[] gridToBytes(GridData grid) {
        if (grid == null) {
            return null;
        }
        MemStream memStream = new MemStream();
        try {
            grid.saveToStream((Stream) memStream);
            return memStream.getBytes();
        } catch (StreamException ex) {
            ex.printStackTrace();
            return null;
        }
    }

    public static byte[] gridToBytes(GridData grid, String encodeCharset) {
        if (grid == null) {
            return null;
        }
        MemStream memStream = new MemStream();
        try {
            memStream.setUseEncode(true);
            memStream.setCharset(encodeCharset);
            grid.saveToStream((Stream) memStream);
            return memStream.getBytes();
        } catch (StreamException ex) {
            ex.printStackTrace();
            return null;
        }
    }

    public static GridData base64ToGrid(String base64Str) {
        byte[] data = Base64.base64ToByteArray(base64Str);
        return bytesToGrid(data);
    }

    public static GridData base64ToGrid(String base64Str, String encodeCharset) {
        byte[] data = Base64.base64ToByteArray(base64Str);
        return bytesToGrid(data, encodeCharset);
    }

    public static String gridToBase64(GridData grid) {
        byte[] data = gridToBytes(grid);
        return Base64.byteArrayToBase64(data);
    }

    public static String gridToBase64(GridData grid, String encodeCharset) {
        byte[] data = gridToBytes(grid, encodeCharset);
        return Base64.byteArrayToBase64(data);
    }

    public void convertData(CellConvertor convertor) {
        for (int col = 0; col < getColCount(); col++) {
            for (int row = 0; row < getRowCount(); row++) {
                CellField cf = expandCell(col, row);
                if (cf.left == col && cf.top == row) {
                    GridCell cell = getCellEx(col, row);
                    if (convertor.convert(this, cell)) {
                        setCell(col, row, cell);
                    }
                }
            }
        }
    }

    public int getOptions() {
        return this.options;
    }

    public void setOptions(int value) {
        this.options = value;
        this.rects.setAutoMerge((value & 1) != 0);
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: com_jiuqi_grid.jar:com/jiuqi/grid/GridData$GridCopyConvertor.class */
    public final class GridCopyConvertor {
        GridData src;
        int[] fontNameMaps;
        int[] fontSizeMaps;
        int[] fontColorMaps;
        int[] cellColorMaps;
        int[] edgeColorMaps;
        private Stack freeFontNames;
        private Stack freeFontSizes;
        private Stack freeFontColors;
        private Stack freeCellColors;
        private Stack freeEdgeColors;

        public GridCopyConvertor(GridData src) {
            this.src = src;
            mergeFontNames(src.fontNames);
            this.fontSizeMaps = mergeList(src.fontSizes, GridData.this.fontSizes);
            this.fontColorMaps = mergeList(src.fontColors, GridData.this.fontColors);
            this.cellColorMaps = mergeList(src.cellColors, GridData.this.cellColors);
            this.edgeColorMaps = mergeList(src.edgeColors, GridData.this.edgeColors);
        }

        private void mergeFontNames(GridFontNames srcFontNames) {
            boolean diff = false;
            int[] maps = new int[srcFontNames.count()];
            for (int i = 0; i < srcFontNames.count(); i++) {
                int index = GridData.this.fontNames.indexOfFont(srcFontNames.getFont(i));
                maps[i] = index;
                if (i != index) {
                    diff = true;
                }
            }
            this.fontNameMaps = diff ? maps : null;
        }

        private int[] mergeList(IntList srcList, IntList destList) {
            boolean diff = false;
            int[] maps = new int[srcList.count()];
            for (int i = 0; i < srcList.count(); i++) {
                int index = destList.indexOf(srcList.get(i));
                maps[i] = index;
                if (i != index) {
                    diff = true;
                }
            }
            if (diff) {
                return maps;
            }
            return null;
        }

        public void convert(GridCell cell) {
            byte[] data = cell.getPropData();
            if (this.fontNameMaps != null) {
                int index = data[4] & 63;
                int newIndex = mapFontName(index);
                data[4] = (byte) ((data[4] & 192) | (newIndex & 63));
            }
            if (this.fontSizeMaps != null) {
                int index2 = data[5] & 63;
                int newIndex2 = mapFontSize(index2);
                data[5] = (byte) ((data[5] & 192) | (newIndex2 & 63));
            }
            if (this.fontColorMaps != null) {
                int index3 = (data[6] >> 0) & 63;
                int newIndex3 = mapFontColor(index3);
                data[6] = (byte) ((data[6] & 192) | (newIndex3 & 63));
            }
            if (this.cellColorMaps != null) {
                int newIndex4 = mapCellColor(data[0]);
                data[0] = (byte) newIndex4;
            }
            if (this.edgeColorMaps != null) {
                int index4 = (data[9] >> 4) & 15;
                int newIndex5 = mapEdgeColor(index4);
                data[9] = (byte) ((data[9] & 15) | ((newIndex5 << 4) & 240));
                int index5 = (data[10] >> 4) & 15;
                int newIndex6 = mapEdgeColor(index5);
                data[10] = (byte) ((data[10] & 15) | ((newIndex6 << 4) & 240));
            }
        }

        private int mapCellColor(int index) {
            int newIndex;
            if (this.cellColorMaps[index] == -1) {
                if (freeCellColors().isEmpty()) {
                    newIndex = GridData.this.cellColors.count() >= 64 ? 0 : GridData.this.cellColors.add(this.src.cellColors.get(index));
                } else {
                    newIndex = ((Integer) freeCellColors().pop()).intValue();
                    GridData.this.cellColors.set(newIndex, this.src.cellColors.get(index));
                }
                this.cellColorMaps[index] = newIndex;
            } else {
                newIndex = this.cellColorMaps[index];
            }
            return newIndex;
        }

        private int mapFontColor(int index) {
            int newIndex;
            if (this.fontColorMaps[index] == -1) {
                if (freeFontColors().isEmpty()) {
                    newIndex = GridData.this.fontColors.count() >= 64 ? 0 : GridData.this.fontColors.add(this.src.fontColors.get(index));
                } else {
                    newIndex = ((Integer) freeFontColors().pop()).intValue();
                    GridData.this.fontColors.set(newIndex, this.src.fontColors.get(index));
                }
                this.fontColorMaps[index] = newIndex;
            } else {
                newIndex = this.fontColorMaps[index];
            }
            return newIndex;
        }

        private int mapFontSize(int index) {
            int newIndex;
            if (this.fontSizeMaps[index] == -1) {
                if (freeFontSizes().isEmpty()) {
                    newIndex = GridData.this.fontSizes.count() >= 64 ? 0 : GridData.this.fontSizes.add(this.src.fontSizes.get(index));
                } else {
                    newIndex = ((Integer) freeFontSizes().pop()).intValue();
                    GridData.this.fontSizes.set(newIndex, this.src.fontSizes.get(index));
                }
                this.fontSizeMaps[index] = newIndex;
            } else {
                newIndex = this.fontSizeMaps[index];
            }
            return newIndex;
        }

        private int mapFontName(int index) {
            int newIndex;
            if (this.fontNameMaps[index] == -1) {
                if (freeFontNames().isEmpty()) {
                    newIndex = GridData.this.fontNames.count() >= 64 ? 0 : GridData.this.fontNames.addFont(this.src.fontNames.getFont(index));
                } else {
                    newIndex = ((Integer) freeFontNames().pop()).intValue();
                    GridData.this.fontNames.setFont(newIndex, this.src.fontNames.getFont(index));
                }
                this.fontNameMaps[index] = newIndex;
            } else {
                newIndex = this.fontNameMaps[index];
            }
            return newIndex;
        }

        private int mapEdgeColor(int index) {
            int newIndex;
            if (this.edgeColorMaps[index] == -1) {
                if (freeEdgeColors().isEmpty()) {
                    newIndex = GridData.this.edgeColors.count() >= 64 ? 0 : GridData.this.edgeColors.add(this.src.edgeColors.get(index));
                } else {
                    newIndex = ((Integer) freeEdgeColors().pop()).intValue();
                    GridData.this.edgeColors.set(newIndex, this.src.edgeColors.get(index));
                }
                this.edgeColorMaps[index] = newIndex;
            } else {
                newIndex = this.edgeColorMaps[index];
            }
            return newIndex;
        }

        private Stack freeFontNames() {
            if (this.freeFontNames == null) {
                boolean[] flags = new boolean[GridData.this.fontNames.count()];
                for (int i = 0; i < GridData.this.propList.count(); i++) {
                    byte[] data = GridData.this.propList.get(i);
                    int index = data[4] & 63;
                    flags[index] = true;
                }
                this.freeFontNames = new Stack();
                for (int i2 = 0; i2 < flags.length; i2++) {
                    if (!flags[i2]) {
                        this.freeFontNames.push(new Integer(i2));
                    }
                }
            }
            return this.freeFontNames;
        }

        private Stack freeFontSizes() {
            if (this.freeFontSizes == null) {
                boolean[] flags = new boolean[GridData.this.fontSizes.count()];
                for (int i = 0; i < GridData.this.propList.count(); i++) {
                    byte[] data = GridData.this.propList.get(i);
                    int index = data[5] & 63;
                    flags[index] = true;
                }
                this.freeFontSizes = new Stack();
                for (int i2 = 0; i2 < flags.length; i2++) {
                    if (!flags[i2]) {
                        this.freeFontSizes.push(new Integer(i2));
                    }
                }
            }
            return this.freeFontSizes;
        }

        private Stack freeFontColors() {
            if (this.freeFontColors == null) {
                boolean[] flags = new boolean[GridData.this.fontColors.count()];
                for (int i = 0; i < GridData.this.propList.count(); i++) {
                    byte[] data = GridData.this.propList.get(i);
                    int index = (data[6] >> 0) & 63;
                    flags[index] = true;
                }
                this.freeFontColors = new Stack();
                for (int i2 = 0; i2 < flags.length; i2++) {
                    if (!flags[i2]) {
                        this.freeFontColors.push(new Integer(i2));
                    }
                }
            }
            return this.freeFontColors;
        }

        private Stack freeCellColors() {
            if (this.freeCellColors == null) {
                boolean[] flags = new boolean[GridData.this.cellColors.count()];
                for (int i = 0; i < GridData.this.propList.count(); i++) {
                    byte[] data = GridData.this.propList.get(i);
                    flags[data[0]] = true;
                }
                this.freeCellColors = new Stack();
                for (int i2 = 0; i2 < flags.length; i2++) {
                    if (!flags[i2]) {
                        this.freeCellColors.push(new Integer(i2));
                    }
                }
            }
            return this.freeCellColors;
        }

        private Stack freeEdgeColors() {
            if (this.freeEdgeColors == null) {
                boolean[] flags = new boolean[GridData.this.edgeColors.count()];
                for (int i = 0; i < GridData.this.propList.count(); i++) {
                    byte[] data = GridData.this.propList.get(i);
                    int index = (data[9] >> 4) & 15;
                    flags[index] = true;
                    int index2 = (data[10] >> 4) & 15;
                    flags[index2] = true;
                }
                this.freeEdgeColors = new Stack();
                for (int i2 = 0; i2 < flags.length; i2++) {
                    if (!flags[i2]) {
                        this.freeEdgeColors.push(new Integer(i2));
                    }
                }
            }
            return this.freeEdgeColors;
        }
    }
}
